@@ -278,6 +278,8 @@ def get_proxies(debug=True):
             continue
         if proxy.startswith(key+'://'):
             proxy = proxy[len(key)+3:]
+        if key == 'https' and proxy.startswith('http://'):
+            proxy = proxy[7:]
         if proxy.endswith('/'):
             proxy = proxy[:-1]
         if len(proxy) > 4:
@@ -320,6 +322,28 @@ def get_parsed_proxy(typ='http', debug=True):
                     prints('Using http proxy', str(ans))
                 return ans
 
+def get_proxy_info(proxy_scheme, proxy_string):
+    '''
+    Parse all proxy information from a proxy string (as returned by
+    get_proxies). The returned dict will have members set to None when the info
+    is not available in the string. If an exception occurs parsing the string
+    this method returns None.
+    '''
+    import urlparse
+    try:
+        proxy_url = u'%s://%s'%(proxy_scheme, proxy_string)
+        urlinfo = urlparse.urlparse(proxy_url)
+        ans = {
+            u'scheme': urlinfo.scheme,
+            u'hostname': urlinfo.hostname,
+            u'port': urlinfo.port,
+            u'username': urlinfo.username,
+            u'password': urlinfo.password,
+        }
+    except:
+        return None
+    return ans
+
 USER_AGENT = 'Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.13) Gecko/20101210 Gentoo Firefox/3.6.13'
 USER_AGENT_MOBILE = 'Mozilla/5.0 (Windows; U; Windows CE 5.1; rv:1.8.1a3) Gecko/20060610 Minimo/0.016'
 
@@ -341,7 +365,7 @@ def random_user_agent():
 def browser(honor_time=True, max_time=2, mobile_browser=False, user_agent=None):
     '''
     Create a mechanize browser for web scraping. The browser handles cookies,
-    refresh requests and ignores robots.txt. Also uses proxy if avaialable.
+    refresh requests and ignores robots.txt. Also uses proxy if available.
 
     :param honor_time: If True honors pause time in refresh requests
     :param max_time: Maximum time in seconds to wait during a refresh request
@@ -353,9 +377,14 @@ def browser(honor_time=True, max_time=2, mobile_browser=False, user_agent=None):
     if user_agent is None:
         user_agent = USER_AGENT_MOBILE if mobile_browser else USER_AGENT
     opener.addheaders = [('User-agent', user_agent)]
-    http_proxy = get_proxies().get('http', None)
+    proxies = get_proxies()
+    http_proxy = proxies.get('http', None)
     if http_proxy:
         opener.set_proxies({'http':http_proxy})
+    https_proxy = proxies.get('https', None)
+    if https_proxy:
+        opener.set_proxies({'https':https_proxy})
+
     return opener
 
 def fit_image(width, height, pwidth, pheight):
@@ -474,7 +503,7 @@ def strftime(fmt, t=None):
 def my_unichr(num):
     try:
         return unichr(num)
-    except ValueError:
+    except (ValueError, OverflowError):
         return u'?'
 
 def entity_to_unicode(match, exceptions=[], encoding='cp1252',
