@@ -49,6 +49,9 @@ class DevicePlugin(Plugin):
     #: Whether the metadata on books can be set via the GUI.
     CAN_SET_METADATA = ['title', 'authors', 'collections']
 
+    #: Whether the device can handle device_db metadata plugboards
+    CAN_DO_DEVICE_DB_PLUGBOARD = False
+
     # Set this to None if the books on the device are files that the GUI can
     # access in order to add the books from the device to the library
     BACKLOADING_ERROR_MESSAGE = _('Cannot get files from this device')
@@ -59,7 +62,7 @@ class DevicePlugin(Plugin):
     #: Icon for this device
     icon = I('reader.png')
 
-    # Used by gui2.ui:annotations_fetched() and devices.kindle.driver:get_annotations()
+    # Encapsulates an annotation fetched from the device
     UserAnnotation = namedtuple('Annotation','type, value')
 
     #: GUI displays this as a message if not None. Useful if opening can take a
@@ -71,6 +74,12 @@ class DevicePlugin(Plugin):
     #: For example: ``frozenset(['kobo'])``
     VIRTUAL_BOOK_EXTENSIONS = frozenset([])
 
+    #: Whether to nuke comments in the copy of the book sent to the device. If
+    #: not None this should be short string that the comments will be replaced
+    #: by.
+    NUKE_COMMENTS = None
+
+
     @classmethod
     def get_gui_name(cls):
         if hasattr(cls, 'gui_name'):
@@ -208,7 +217,7 @@ class DevicePlugin(Plugin):
         '''
         Unix version of :meth:`can_handle_windows`
 
-        :param device_info: Is a tupe of (vid, pid, bcd, manufacturer, product,
+        :param device_info: Is a tuple of (vid, pid, bcd, manufacturer, product,
                             serial number)
 
         '''
@@ -405,7 +414,8 @@ class DevicePlugin(Plugin):
     @classmethod
     def config_widget(cls):
         '''
-        Should return a QWidget. The QWidget contains the settings for the device interface
+        Should return a QWidget. The QWidget contains the settings for the
+        device interface
         '''
         raise NotImplementedError()
 
@@ -420,8 +430,9 @@ class DevicePlugin(Plugin):
     @classmethod
     def settings(cls):
         '''
-        Should return an opts object. The opts object should have at least one attribute
-        `format_map` which is an ordered list of formats for the device.
+        Should return an opts object. The opts object should have at least one
+        attribute `format_map` which is an ordered list of formats for the
+        device.
         '''
         raise NotImplementedError()
 
@@ -453,6 +464,13 @@ class DevicePlugin(Plugin):
         '''
         pass
 
+    def prepare_addable_books(self, paths):
+        '''
+        Given a list of paths, returns another list of paths. These paths
+        point to addable versions of the books.
+        '''
+        return paths
+
 class BookList(list):
     '''
     A list of books. Each Book object must have the fields
