@@ -483,7 +483,7 @@ class Device(DeviceConfig, DevicePlugin):
         self._card_a_prefix = get_card_prefix('carda')
         self._card_b_prefix = get_card_prefix('cardb')
 
-    def find_device_nodes(self):
+    def find_device_nodes(self, detected_device=None):
 
         def walk(base):
             base = os.path.abspath(os.path.realpath(base))
@@ -507,8 +507,11 @@ class Device(DeviceConfig, DevicePlugin):
         d, j = os.path.dirname, os.path.join
         usb_dir = None
 
+        if detected_device is None:
+            detected_device = self.detected_device
+
         def test(val, attr):
-            q = getattr(self.detected_device, attr)
+            q = getattr(detected_device, attr)
             return q == val
 
         for x, isfile in walk('/sys/devices'):
@@ -596,6 +599,8 @@ class Device(DeviceConfig, DevicePlugin):
                 label = self.STORAGE_CARD2_VOLUME_LABEL
                 if not label:
                     label = self.STORAGE_CARD_VOLUME_LABEL + ' 2'
+            if not label:
+                label = 'E-book Reader (%s)'%type
             extra = 0
             while True:
                 q = ' (%d)'%extra if extra else ''
@@ -611,20 +616,11 @@ class Device(DeviceConfig, DevicePlugin):
                     mount(node)
                     return 0
                 except:
-                    pass
+                    print 'Udisks mount call failed:'
+                    import traceback
+                    traceback.print_exc()
+                    return 1
 
-                cmd = 'calibre-mount-helper'
-                if getattr(sys, 'frozen', False):
-                    cmd = os.path.join(sys.executables_location, 'bin', cmd)
-                cmd = [cmd, 'mount']
-                try:
-                    p = subprocess.Popen(cmd + [node, '/media/'+label])
-                except OSError:
-                    raise DeviceError(
-                    _('Could not find mount helper: %s.')%cmd[0])
-                while p.poll() is None:
-                    time.sleep(0.1)
-                return p.returncode
 
             ret = do_mount(node, label)
             if ret != 0:
@@ -772,9 +768,12 @@ class Device(DeviceConfig, DevicePlugin):
             # try all the nodes to see what we can mount
             for dev in devs[i].split():
                 mp='/media/'+label+'-'+dev
+                mmp = mp
+                if mmp.endswith('/'):
+                    mmp = mmp[:-1]
                 #print "trying ", dev, "on", mp
                 try:
-                    p = subprocess.Popen(cmd + ["/dev/"+dev, mp])
+                    p = subprocess.Popen(cmd + ["/dev/"+dev, mmp])
                 except OSError:
                     raise DeviceError(_('Could not find mount helper: %s.')%cmd[0])
                 while p.poll() is None:
@@ -923,29 +922,12 @@ class Device(DeviceConfig, DevicePlugin):
                 umount(d)
             except:
                 pass
-        failures = False
         for d in drives:
             try:
                 eject(d)
             except Exception as e:
                 print 'Udisks eject call for:', d, 'failed:'
                 print '\t', e
-                failures = True
-
-        if not failures:
-            return
-
-        for drive in drives:
-            cmd = 'calibre-mount-helper'
-            if getattr(sys, 'frozen', False):
-                cmd = os.path.join(sys.executables_location, 'bin', cmd)
-            cmd = [cmd, 'eject']
-            mp = getattr(self, "_linux_mount_map", {}).get(drive,
-                    'dummy/')[:-1]
-            try:
-                subprocess.Popen(cmd + [drive, mp]).wait()
-            except:
-                pass
 
     def eject(self):
         if islinux:
@@ -971,19 +953,6 @@ class Device(DeviceConfig, DevicePlugin):
         self._main_prefix = self._card_a_prefix = self._card_b_prefix = None
 
     def linux_post_yank(self):
-        for drive, mp in getattr(self, '_linux_mount_map', {}).items():
-            if drive and mp:
-                mp = mp[:-1]
-                cmd = 'calibre-mount-helper'
-                if getattr(sys, 'frozen', False):
-                    cmd = os.path.join(sys.executables_location, 'bin', cmd)
-                cmd = [cmd, 'cleanup']
-                if mp and os.path.exists(mp):
-                    try:
-                        subprocess.Popen(cmd + [drive, mp]).wait()
-                    except:
-                        import traceback
-                        traceback.print_exc()
         self._linux_mount_map = {}
 
     def post_yank_cleanup(self):
@@ -1063,6 +1032,12 @@ class Device(DeviceConfig, DevicePlugin):
         '''
         return {}
 
+    def add_annotation_to_library(self, db, db_id, annotation):
+        '''
+        Add an annotation to the calibre library
+        '''
+        pass
+
     def create_upload_path(self, path, mdata, fname, create_dirs=True):
         path = os.path.abspath(path)
         maxlen = self.MAX_PATH_LEN
@@ -1077,8 +1052,13 @@ class Device(DeviceConfig, DevicePlugin):
         settings = self.settings()
         template = self.save_template()
         if mdata.tags and _('News') in mdata.tags:
-            today = time.localtime()
-            template = "{title}_%d-%d-%d" % (today[0], today[1], today[2])
+            try:
+                p = mdata.pubdate
+                date  = (p.year, p.month, p.day)
+            except:
+                today = time.localtime()
+                date = (today[0], today[1], today[2])
+            template = "{title}_%d-%d-%d" % date
         use_subdirs = self.SUPPORTS_SUB_DIRS and settings.use_subdirs
 
         fname = sanitize(fname)
@@ -1137,3 +1117,6 @@ class Device(DeviceConfig, DevicePlugin):
             os.makedirs(filedir)
 
         return filepath
+
+    def create_annotations_path(self, mdata, device_path=None):
+         return self.create_upload_path(os.path.abspath('/<storage>'), mdata, 'x.bookmark', create_dirs=False)
