@@ -265,6 +265,14 @@ OptionRecommendation(name='toc_filter',
                 )
         ),
 
+OptionRecommendation(name='duplicate_links_in_toc',
+            recommended_value=False, level=OptionRecommendation.LOW,
+            help=_('When creating a TOC from links in the input document, '
+                'allow duplicate entries, i.e. allow more than one entry '
+                'with the same text, provided that they point to a '
+                'different location.')
+        ),
+
 
 OptionRecommendation(name='chapter',
         recommended_value="//*[((name()='h1' or name()='h2') and "
@@ -358,9 +366,11 @@ OptionRecommendation(name='remove_paragraph_spacing',
 
 OptionRecommendation(name='remove_paragraph_spacing_indent_size',
         recommended_value=1.5, level=OptionRecommendation.LOW,
-        help=_('When calibre removes inter paragraph spacing, it automatically '
+        help=_('When calibre removes blank lines between paragraphs, it automatically '
             'sets a paragraph indent, to ensure that paragraphs can be easily '
-            'distinguished. This option controls the width of that indent.')
+            'distinguished. This option controls the width of that indent (in em). '
+            'If you set this value negative, then the indent specified in the input '
+            'document is used, that is, calibre does not change the indentation.')
         ),
 
 OptionRecommendation(name='prefer_metadata_cover',
@@ -376,11 +386,19 @@ OptionRecommendation(name='insert_blank_line',
             )
         ),
 
+OptionRecommendation(name='insert_blank_line_size',
+        recommended_value=0.5, level=OptionRecommendation.LOW,
+        help=_('Set the height of the inserted blank lines (in em).'
+            ' The height of the lines between paragraphs will be twice the value'
+            ' set here.')
+        ),
+
 OptionRecommendation(name='remove_first_image',
         recommended_value=False, level=OptionRecommendation.LOW,
         help=_('Remove the first image from the input ebook. Useful if the '
-        'first image in the source file is a cover and you are specifying '
-        'an external cover.'
+        'input document has a cover image that is not identified as a cover. '
+        'In this case, if you set a cover in calibre, the output document will '
+        'end up with two cover images if you do not specify this option.'
             )
         ),
 
@@ -400,6 +418,13 @@ OptionRecommendation(name='smarten_punctuation',
             )
         ),
 
+OptionRecommendation(name='unsmarten_punctuation',
+        recommended_value=False, level=OptionRecommendation.LOW,
+        help=_('Convert fancy quotes, dashes and ellipsis to their '
+               'plain equivalents.'
+            )
+        ),
+
 OptionRecommendation(name='read_metadata_from_opf',
             recommended_value=None, level=OptionRecommendation.LOW,
             short_switch='m',
@@ -542,7 +567,7 @@ OptionRecommendation(name='delete_blank_paragraphs',
 OptionRecommendation(name='format_scene_breaks',
     recommended_value=True, level=OptionRecommendation.LOW,
     help=_('Left aligned scene break markers are center aligned. '
-           'Replace soft scene breaks that use multiple blank lines with'
+           'Replace soft scene breaks that use multiple blank lines with '
            'horizontal rules.')),
 
 OptionRecommendation(name='replace_scene_breaks',
@@ -594,7 +619,7 @@ OptionRecommendation(name='sr3_replace',
         input_fmt = os.path.splitext(self.input)[1]
         if not input_fmt:
             raise ValueError('Input file must have an extension')
-        input_fmt = input_fmt[1:].lower()
+        input_fmt = input_fmt[1:].lower().replace('original_', '')
         self.archive_input_tdir = None
         if input_fmt in ARCHIVE_FMTS:
             self.log('Processing archive...')
@@ -668,6 +693,8 @@ OptionRecommendation(name='sr3_replace',
     def unarchive(self, path, tdir):
         extract(path, tdir)
         files = list(walk(tdir))
+        files = [f if isinstance(f, unicode) else f.decode(filesystem_encoding)
+                for f in files]
         from calibre.customize.ui import available_input_formats
         fmts = available_input_formats()
         for x in ('htm', 'html', 'xhtm', 'xhtml'): fmts.remove(x)
@@ -969,7 +996,11 @@ OptionRecommendation(name='sr3_replace',
         if fkey is None:
             fkey = self.opts.dest.fkey
         else:
-            fkey = map(float, fkey.split(','))
+            try:
+                fkey = map(float, fkey.split(','))
+            except:
+                self.log.error('Invalid font size key: %r ignoring'%fkey)
+                fkey = self.opts.dest.fkey
 
         from calibre.ebooks.oeb.transforms.jacket import Jacket
         Jacket()(self.oeb, self.opts, self.user_metadata)
@@ -999,6 +1030,10 @@ OptionRecommendation(name='sr3_replace',
             from calibre.ebooks.oeb.transforms.linearize_tables import LinearizeTables
             LinearizeTables()(self.oeb, self.opts)
 
+        if self.opts.unsmarten_punctuation:
+            from calibre.ebooks.oeb.transforms.unsmarten import UnsmartenPunctuation
+            UnsmartenPunctuation()(self.oeb, self.opts)
+
         flattener = CSSFlattener(fbase=fbase, fkey=fkey,
                 lineh=line_height,
                 untable=self.output_plugin.file_type in ('mobi','lit'),
@@ -1040,6 +1075,7 @@ OptionRecommendation(name='sr3_replace',
         with self.output_plugin:
             self.output_plugin.convert(self.oeb, self.output, self.input_plugin,
                 self.opts, self.log)
+        self.oeb.clean_temp_files()
         self.ui_reporter(1.)
         run_plugins_on_postprocess(self.output, self.output_fmt)
 
