@@ -20,6 +20,7 @@ from calibre.utils.config import DynamicConfig
 from calibre.utils.logging import Log
 from calibre import guess_type, prints, prepare_string_for_xml
 from calibre.ebooks.oeb.transforms.cover import CoverManager
+from calibre.constants import filesystem_encoding
 
 TITLEPAGE = CoverManager.SVG_TEMPLATE.decode('utf-8').replace(\
         '__ar__', 'none').replace('__viewbox__', '0 0 600 800'
@@ -92,7 +93,7 @@ class EbookIterator(object):
         self.config = DynamicConfig(name='iterator')
         ext = os.path.splitext(pathtoebook)[1].replace('.', '').lower()
         ext = re.sub(r'(x{0,1})htm(l{0,1})', 'html', ext)
-        self.ebook_ext = ext
+        self.ebook_ext = ext.replace('original_', '')
 
     def search(self, text, index, backwards=False):
         text = text.lower()
@@ -180,6 +181,8 @@ class EbookIterator(object):
         self.delete_on_exit = []
         self._tdir = TemporaryDirectory('_ebook_iter')
         self.base  = self._tdir.__enter__()
+        if not isinstance(self.base, unicode):
+            self.base = self.base.decode(filesystem_encoding)
         from calibre.ebooks.conversion.plumber import Plumber, create_oebbook
         plumber = Plumber(self.pathtoebook, self.base, self.log)
         plumber.setup_options()
