@@ -246,6 +246,7 @@ class CSSFlattener(object):
                     cssdict['font-size'] = '%.1fpt'%font_size
                 del node.attrib['size']
             if 'face' in node.attrib:
+                cssdict['font-family'] = node.attrib['face']
                 del node.attrib['face']
         if 'color' in node.attrib:
             cssdict['color'] = node.attrib['color']
@@ -318,9 +319,13 @@ class CSSFlattener(object):
                     for edge in ('top', 'bottom'):
                         cssdict['%s-%s'%(prop, edge)] = '0pt'
             if self.context.insert_blank_line:
-                cssdict['margin-top'] = cssdict['margin-bottom'] = '0.5em'
-            if self.context.remove_paragraph_spacing:
-                cssdict['text-indent'] =  "%1.1fem" % self.context.remove_paragraph_spacing_indent_size
+                cssdict['margin-top'] = cssdict['margin-bottom'] = \
+                    '%fem'%self.context.insert_blank_line_size
+            indent_size = self.context.remove_paragraph_spacing_indent_size
+            keep_indents = indent_size < 0.0
+            if (self.context.remove_paragraph_spacing and not keep_indents and
+                cssdict.get('text-align', None) not in ('center', 'right')):
+                cssdict['text-indent'] =  "%1.1fem" % indent_size
 
         if cssdict:
             items = cssdict.items()
