@@ -87,7 +87,7 @@ class DeviceJob(BaseJob): # {{{
             self.failed = True
             ex = as_unicode(err)
             self._details = ex + '\n\n' + \
-                traceback.format_exc()
+                force_unicode(traceback.format_exc())
             self.exception = err
         finally:
             self.job_done()
@@ -391,6 +391,10 @@ class DeviceManager(Thread): # {{{
                                     newmi.template_to_attribute(mi, cpb)
                                 else:
                                     newmi = mi
+                                nuke_comments = getattr(self.connected_device,
+                                        'NUKE_COMMENTS', None)
+                                if nuke_comments is not None:
+                                    mi.comments = nuke_comments
                                 set_metadata(stream, newmi, stream_type=ext)
                         except:
                             if DEBUG:
@@ -846,15 +850,16 @@ class DeviceMixin(object): # {{{
         self.refresh_ondevice()
         device_signals.device_metadata_available.emit()
 
-    def refresh_ondevice(self, reset_only = False):
+    def refresh_ondevice(self, reset_only=False):
         '''
         Force the library view to refresh, taking into consideration new
         device books information
         '''
-        self.book_on_device(None, reset=True)
-        if reset_only:
-            return
-        self.library_view.model().refresh_ondevice()
+        with self.library_view.preserve_state():
+            self.book_on_device(None, reset=True)
+            if reset_only:
+                return
+            self.library_view.model().refresh_ondevice()
 
     # }}}
 
@@ -884,7 +889,6 @@ class DeviceMixin(object): # {{{
         # if set_books_in_library did not.
         if not self.set_books_in_library(self.booklists(), reset=True, add_as_step_to_job=job):
             self.upload_booklists(job)
-        self.book_on_device(None, reset=True)
         # We need to reset the ondevice flags in the library. Use a big hammer,
         # so we don't need to worry about whether some succeeded or not.
         self.refresh_ondevice(reset_only=False)
@@ -912,8 +916,9 @@ class DeviceMixin(object): # {{{
                             format_count[f] = 1
             for f in self.device_manager.device.settings().format_map:
                 if f in format_count.keys():
-                    formats.append((f, _('%i of %i Books') % (format_count[f],
-                        len(rows)), True if f in aval_out_formats else False))
+                    formats.append((f, _('%(num)i of %(total)i Books') % dict(
+                        num=format_count[f], total=len(rows)),
+                        True if f in aval_out_formats else False))
                 elif f in aval_out_formats:
                     formats.append((f, _('0 of %i Books') % len(rows), True))
             d = ChooseFormatDeviceDialog(self, _('Choose format to send to device'), formats)
@@ -1314,9 +1319,7 @@ class DeviceMixin(object): # {{{
         # If it does not, then do it here.
         if not self.set_books_in_library(self.booklists(), reset=True, add_as_step_to_job=job):
             self.upload_booklists(job)
-        with self.library_view.preserve_selected_books:
-            self.book_on_device(None, reset=True)
-            self.refresh_ondevice()
+        self.refresh_ondevice()
 
         view = self.card_a_view if on_card == 'carda' else \
             self.card_b_view if on_card == 'cardb' else self.memory_view
