@@ -21,7 +21,7 @@ from calibre.utils.ipc.job import ParallelJob
 from calibre.gui2 import Dispatcher, error_dialog, question_dialog, NONE, config, gprefs
 from calibre.gui2.device import DeviceJob
 from calibre.gui2.dialogs.jobs_ui import Ui_JobsDialog
-from calibre import __appname__
+from calibre import __appname__, as_unicode
 from calibre.gui2.dialogs.job_view_ui import Ui_Dialog
 from calibre.gui2.progress_indicator import ProgressIndicator
 from calibre.gui2.threaded_jobs import ThreadedJobServer, ThreadedJob
@@ -172,14 +172,29 @@ class JobManager(QAbstractTableModel): # {{{
                     if job.is_finished:
                         self.job_done.emit(len(self.unfinished_jobs()))
             if needs_reset:
+                self.layoutAboutToBeChanged.emit()
                 self.jobs.sort()
-                self.reset()
+                self.layoutChanged.emit()
             else:
                 for job in jobs:
                     idx = self.jobs.index(job)
                     self.dataChanged.emit(
                         self.index(idx, 0), self.index(idx, 3))
 
+        # Kill parallel jobs that have gone on too long
+        try:
+            wmax_time = gprefs['worker_max_time'] * 60
+        except:
+            wmax_time = 0
+
+        if wmax_time > 0:
+            for job in self.jobs:
+                if isinstance(job, ParallelJob):
+                    rtime = job.running_time
+                    if (rtime is not None and rtime > wmax_time and
+                            job.duration is None):
+                        job.timed_out = True
+                        self.server.kill_job(job)
 
     def _add_job(self, job):
         self.layoutAboutToBeChanged.emit()
@@ -249,6 +264,26 @@ class JobManager(QAbstractTableModel): # {{{
                     _('This job cannot be stopped'), show=True)
         self._kill_job(job)
 
+    def kill_multiple_jobs(self, rows, view):
+        jobs = [self.jobs[row] for row in rows]
+        devjobs = [j for j in jobs if isinstance(j, DeviceJob)]
+        if devjobs:
+            error_dialog(view, _('Cannot kill job'),
+                         _('Cannot kill jobs that communicate with the device')).exec_()
+            jobs = [j for j in jobs if not isinstance(j, DeviceJob)]
+        jobs = [j for j in jobs if j.duration is None]
+        unkillable = [j for j in jobs if not getattr(j, 'killable', True)]
+        if unkillable:
+            names = u'\n'.join(as_unicode(j.description) for j in unkillable)
+            error_dialog(view, _('Cannot kill job'),
+                    _('Some of the jobs cannot be stopped. Click Show details'
+                        ' to see the list of unstoppable jobs.'), det_msg=names,
+                    show=True)
+            jobs = [j for j in jobs if getattr(j, 'killable', True)]
+        jobs = [j for j in jobs if j.duration is None]
+        for j in jobs:
+            self._kill_job(j)
+
     def kill_all_jobs(self):
         for job in self.jobs:
             if (isinstance(job, DeviceJob) or job.duration is not None or
@@ -267,7 +302,8 @@ class JobManager(QAbstractTableModel): # {{{
 # }}}
 
 # Jobs UI {{{
-class ProgressBarDelegate(QAbstractItemDelegate):
+
+class ProgressBarDelegate(QAbstractItemDelegate): # {{{
 
     def sizeHint(self, option, index):
         return QSize(120, 30)
@@ -284,8 +320,9 @@ class ProgressBarDelegate(QAbstractItemDelegate):
         opts.progress = percent
         opts.text = QString(_('Unavailable') if percent == 0 else '%d%%'%percent)
         QApplication.style().drawControl(QStyle.CE_ProgressBar, opts, painter)
+# }}}
 
-class DetailView(QDialog, Ui_Dialog):
+class DetailView(QDialog, Ui_Dialog): # {{{
 
     def __init__(self, parent, job):
         QDialog.__init__(self, parent)
@@ -318,8 +355,9 @@ class DetailView(QDialog, Ui_Dialog):
             self.next_pos = f.tell()
             if more:
                 self.log.appendPlainText(more.decode('utf-8', 'replace'))
+# }}}
 
-class JobsButton(QFrame):
+class JobsButton(QFrame): # {{{
 
     def __init__(self, horizontal=False, size=48, parent=None):
         QFrame.__init__(self, parent)
@@ -404,6 +442,7 @@ class JobsButton(QFrame):
             self.stop()
             QCoreApplication.instance().alert(self, 5000)
 
+# }}}
 
 class JobsDialog(QDialog, Ui_JobsDialog):
 
@@ -446,7 +485,6 @@ class JobsDialog(QDialog, Ui_JobsDialog):
         except:
             pass
 
-
     def show_job_details(self, index):
         row = index.row()
         job = self.jobs_view.model().row_to_job(row)
@@ -455,18 +493,28 @@ class JobsDialog(QDialog, Ui_JobsDialog):
         d.timer.stop()
 
     def show_details(self, *args):
-        for index in self.jobs_view.selectedIndexes():
+        index = self.jobs_view.currentIndex()
+        if index.isValid():
             self.show_job_details(index)
-            return
 
     def kill_job(self, *args):
-        if question_dialog(self, _('Are you sure?'), _('Do you really want to stop the selected job?')):
-            for index in self.jobs_view.selectionModel().selectedRows():
-                row = index.row()
-                self.model.kill_job(row, self)
+        rows = [index.row() for index in
+                self.jobs_view.selectionModel().selectedRows()]
+        if not rows:
+            return error_dialog(self, _('No job'),
+                _('No job selected'), show=True)
+        if question_dialog(self, _('Are you sure?'),
+                ngettext('Do you really want to stop the selected job?',
+                    'Do you really want to stop all the selected jobs?',
+                    len(rows))):
+            if len(rows) > 1:
+                self.model.kill_multiple_jobs(rows, self)
+            else:
+                self.model.kill_job(rows[0], self)
 
     def kill_all_jobs(self, *args):
-        if question_dialog(self, _('Are you sure?'), _('Do you really want to stop all non-device jobs?')):
+        if question_dialog(self, _('Are you sure?'),
+                _('Do you really want to stop all non-device jobs?')):
             self.model.kill_all_jobs()
 
     def closeEvent(self, e):
