@@ -47,6 +47,9 @@ def get_parser(usage):
 def get_db(dbpath, options):
     if options.library_path is not None:
         dbpath = options.library_path
+    if dbpath is None:
+        raise ValueError('No saved library path, either run the GUI or use the'
+                ' --with-library option')
     dbpath = os.path.abspath(dbpath)
     return LibraryDatabase2(dbpath)
 
@@ -365,6 +368,7 @@ def command_remove(args, dbpath):
 
 def do_add_format(db, id, fmt, path):
     db.add_format_with_hooks(id, fmt.upper(), path, index_is_id=True)
+    send_message()
 
 def add_format_option_parser():
     return get_parser(_(
@@ -393,6 +397,7 @@ def command_add_format(args, dbpath):
 
 def do_remove_format(db, id, fmt):
     db.remove_format(id, fmt, index_is_id=True)
+    send_message()
 
 def remove_format_option_parser():
     return get_parser(_(
@@ -1040,7 +1045,7 @@ information is the equivalent of what is shown in the tags pane.
     parser.add_option('-r', '--categories', default='', dest='report',
                       help=_("Comma-separated list of category lookup names.\n"
                              "Default: all"))
-    parser.add_option('-w', '--idth', default=-1, type=int,
+    parser.add_option('-w', '--width', default=-1, type=int,
                       help=_('The maximum width of a single line in the output. '
                              'Defaults to detecting screen size.'))
     parser.add_option('-s', '--separator', default=',',
@@ -1097,7 +1102,7 @@ def command_list_categories(args, dbpath):
             for j, field in enumerate(fields):
                 widths[j] = max(widths[j], max(len(field), len(unicode(i[field]))))
 
-        screen_width = terminal_controller.COLS if opts.line_width < 0 else opts.line_width
+        screen_width = terminal_controller.COLS if opts.width < 0 else opts.width
         if not screen_width:
             screen_width = 80
         field_width = screen_width//len(fields)
